@@ -20,15 +20,14 @@
 #' similarity with the reference image (typically representing the spatial
 #' distribution of the signal source) are retrieved. For consistency, the NMI are
 #' scaled in [-1, 1] to match the same range of correlations.
-#'
+#' @param cores integer (default = 1). Number of cores for parallel computing.
 #' @param verbose logical (default = \code{TRUE}). Additional output text.
 #'
 #' @return \code{peak.filter} object. See link{applyPeaksFilter}.
 #'
 #' @details A filter based on the similarity between the peak signals and a reference
-#' signal. The reference signal, passed as an \code{\link{ms.image-class}} object,
-#' can be calculated using the \code{\link{refAndROIimages}} function. Both continuous
-#' and binary references can be passed. The filter then calculates the similarity
+#' signal. The reference signal, passed as an \code{\link{ms.image-class}} object.
+#' Both continuous and binary references can be passed. The filter then calculates the similarity
 #' between the peaks signal and the reference image and select those with a similarity
 #' larger than \code{threshold}. Multiple measures are available, correlation,
 #' structural similarity index measure (SSIM), and normalized mutual information (NMI).
@@ -41,24 +40,25 @@
 #' IEEE transactions on image processing, 13(4), 600-612.
 #' @references Meyer, P. E. (2009). Infotheo: information-theoretic measures.
 #' R package. Version, 1(0).
-#'
+#' 
+#' @import doSNOW foreach
+#' @importFrom utils setTxtProgressBar txtProgressBar
+#' 
 #' @example R/examples/filter_global.R
 #'
 #' @seealso \code{\link{countPixelsFilter}} \code{\link{applyPeaksFilter-msi.dataset-method}}
+#' 
 #' @export
 #'
 globalPeaksFilter <- function(msiData,
                               referenceImage,
                               method = "pearson",
                               threshold = NULL,
+                              cores = 1,
                               verbose = TRUE) {
   .stopIfNotValidMSIDataset(msiData)
   .stopIfNotValidMSImage(referenceImage)
   .stopIfNotValidGlobalMethod(method)
-
-  if (threshold < -1 || threshold > 1) {
-    stop("threshold must be in [-1, 1].")
-  }
   
   if (.isBinary(referenceImage) && method == "pearson") {
     warning("For binary reference images, it is suggested to use the other available methods.\n")
@@ -72,31 +72,53 @@ globalPeaksFilter <- function(msiData,
       threshold <- 0
     }
   }
+  
+  if (threshold < -1 || threshold > 1) {
+    stop("threshold must be in [-1, 1].")
+  }
 
   # Calculate the similarity between the ion images and the reference image
   if (verbose) {
     cat("Calculating the similarity values...\n")
   }
 
-  ## Use NMI only if the reference image is binary
+  # Use NMI only if the reference image is binary
   if (method == "nmi" && !.isBinary(referenceImage)) {
     stop("globalPeaksFilter: 'nmi' can be only used with binary reference images.")
   }
+  
+  method.func <- switch(method,
+                        "pearson" = function(x, y) { cor(x, y, method = "pearson") },
+                        "spearman" = function(x, y) { cor(x, y, method = "spearman") },
+                        "ssim" = function(x, y) { SSIM(x, y) },
+                        "nmi" = function(x, y) { NMI(x, y) })
 
-  r <- array(NA, ncol(msiData@matrix))
-  idx.non.const <- apply(msiData@matrix, 2, var) != 0
+  niter <- ncol(msiData@matrix)
+  pb <- txtProgressBar(max = niter, style = 3, width = 80)
+  progress <- function(n) setTxtProgressBar(pb, n)
+  opts <- list(progress = progress)
+  
+  ref.values <- c(referenceImage@values)
+  
+  if (cores > 1) {
 
-  r[idx.non.const] <- switch(method,
-    "pearson" = apply(msiData@matrix[, idx.non.const], 2, function(z)
-      cor(z, c(referenceImage@values), method = method)),
-    "spearman" = apply(msiData@matrix[, idx.non.const], 2, function(z)
-      cor(z, c(referenceImage@values), method = method)),
-    "ssim" = apply(msiData@matrix[, idx.non.const], 2, function(z)
-      SSIM(z, referenceImage@values)),
-    "nmi" = apply(msiData@matrix[, idx.non.const], 2, function(z)
-      NMI(z, c(referenceImage@values)))
-  )
+    cl <- makeCluster(cores)
+    doSNOW::registerDoSNOW(cl)
+    
+    i <- NULL
+    
+    r <- foreach::foreach(i = 1:niter, .combine = c, .options.snow = opts) %dopar% {
+      method.func(msiData@matrix[, i], ref.values)
+    }
+    
+    stopCluster(cl)
 
+  } else {
+    r <- apply(msiData@matrix, 2, function(z) method.func(z, ref.values))
+  }
+
+  close(pb)
+  
   if (verbose) {
     cat("Similarity measure quantiles (after removing NAs):\n")
     print(quantile(r, na.rm = TRUE))
